#include "AneuMeshLoader.h"
#include "Exceptions.h"
#include <iostream>
#include <fstream>

void AneuMeshLoader::LoadMesh(const std::string& name_file) {
    std::ifstream ifsFile(name_file);
    if (!ifsFile) throw NotFoundFiles();
    else std::cout << "Open file " << name_file << " successfully" << std::endl;

    int number_of_nodes, dimension, number_of_elements, nodes_in_element, number_of_bounds, nodes_in_bound;
    ifsFile >> number_of_nodes >> dimension;
    nodes.reserve(number_of_nodes);
    for (int i = 1; i <= number_of_nodes; ++i) {
        Node t_node;
        t_node.node_id = i;
        ifsFile >> get<0>(t_node.coors) >> get<1>(t_node.coors) >> get<2>(t_node.coors);
        t_node.vertex = false;
        nodes.push_back(t_node);
    }

    ifsFile >> number_of_elements >> nodes_in_element;
    finite_elements.reserve(number_of_elements);
    for (int i = 1; i <= number_of_elements; ++i) {
        FiniteElement t_element;
        t_element.element_id = i;
        ifsFile >> t_element.material_id;
        t_element.nodes_id.reserve(nodes_in_element);
        int temp_id;
        for (int j = 0; j < nodes_in_element; ++j) {
            ifsFile >> temp_id;
            t_element.nodes_id.push_back(temp_id);
        }
        finite_elements.push_back(std::move(t_element)); 
    }

    ifsFile >> number_of_bounds >> nodes_in_bound;
    boundary.reserve(number_of_bounds);
    for (int i = 1; i <= number_of_bounds; ++i) {
        BoundaryFiniteElement t_boundary; 
        t_boundary.element_id = i;
        ifsFile >> t_boundary.surface_id;
        t_boundary.nodes_id.reserve(nodes_in_bound);
        int node_id;
        for (int j = 0; j < dimension; ++j) {
            ifsFile >> node_id;
            nodes[node_id - 1].vertex = true;
            t_boundary.nodes_id.push_back(node_id);
        }
       
        boundary.push_back(std::move(t_boundary));
    
    }
    ifsFile.close();
    std::cout << "Close file " << name_file << " successfully" << std::endl;
}