#include "DataStructures.h"
#include <iterator>
#include <optional>


std::optional<int> readInt(std::istream& in) {
	int i;
	if (in >> i) return { i };
	return {};
}


Node::Node(int t_node_id,const std::array<double,3>& t_coors, bool t_vertex)
	:node_id(t_node_id)
	, coors(t_coors)
	, vertex(t_vertex)
{}

Node::Node(const Node& t_node)
	: node_id(t_node.node_id)
	, coors(t_node.coors)
	, vertex(t_node.vertex)
{}


std::istream& operator >> (std::istream& t_in, Node& t_node)
{
	t_in >> t_node.node_id;
	t_in >> get<0>(t_node.coors) >> get<1>(t_node.coors) >> get<2>(t_node.coors) >> t_node.vertex;
	return t_in;
}

std::ostream& operator<< (std::ostream& t_out, const Node& t_node)
{
	t_out << t_node.node_id;
	t_out << get<0>(t_node.coors) << get<1>(t_node.coors) << get<2>(t_node.coors) << t_node.vertex;
	return t_out;
}

std::istream& operator>>(std::istream& t_in, Edge& t_edge)
{
	t_in >> t_edge.nodes.first >> t_edge.nodes.second;
	return t_in;
}

std::ostream& operator<<(std::ostream& t_out, Edge& t_edge)
{
	t_out << "[" << t_edge.nodes.first << ", " << t_edge.nodes.second << "]";
	return t_out;
}



std::istream& operator>>(std::istream& t_in, FiniteElement& t_finite_elem)
{
	t_in >> t_finite_elem.element_id >> t_finite_elem.material_id;

	std::optional<int> t_node_id;
	t_node_id = readInt(t_in);
	while (t_node_id) {
		t_finite_elem.nodes_id.push_back(*t_node_id);
		t_node_id = readInt(t_in);
	}
	return t_in;
}

std::ostream& operator<<(std::ostream& t_out, const FiniteElement& t_finite_elem)
{
	t_out << t_finite_elem.element_id << std::endl<< t_finite_elem.material_id << std::endl;
	t_out << "[ ";
	for (const auto& elem : t_finite_elem.nodes_id) t_out << elem << " ";
	t_out << "]";
	return t_out;
}

FiniteElement::FiniteElement(int t_element_id, int t_material_id, const std::vector<int>& t_node_id)
	: element_id(t_element_id)
	, material_id(t_material_id)
	, nodes_id(t_node_id)
{}



std::istream& operator>>(std::istream& t_in, BoundaryFiniteElement& t_bound_finite_elem)
{
	t_in >> t_bound_finite_elem.surface_id >> t_bound_finite_elem.element_id;

	std::optional<int> t_node_id;
	t_node_id = readInt(t_in);
	while (t_node_id) {
		t_bound_finite_elem.nodes_id.push_back(*t_node_id);
		t_node_id = readInt(t_in);
	}
	return t_in;
}

std::ostream& operator<<(std::ostream& t_out, const BoundaryFiniteElement& t_bound_finite_elem)
{
	t_out << t_bound_finite_elem.surface_id << std::endl << t_bound_finite_elem.element_id << std::endl;
	t_out << "[ ";
	for (const auto& elem : t_bound_finite_elem.nodes_id) t_out << elem << " ";
	t_out << "]";
	return t_out;
}

BoundaryFiniteElement::BoundaryFiniteElement(int t_element_id, int t_bound_id, const std::vector<int>& t_node_id)
	: element_id(t_element_id)
	, surface_id(t_bound_id)
	, nodes_id(t_node_id)
{}

Edge::Edge(int t_id_first_node, int t_id_second_node)
	: nodes(std::make_pair(t_id_first_node, t_id_second_node))
{}

Edge Edge::operator=(const Edge& t_edge) //!!! Неверный тип возвращаемого значения.
{
    //!!! Где проверка на самоприсваивание? 
    
	Edge res;
	res.nodes = t_edge.nodes;
	return res;
}

bool Edge::operator==(const Edge& t_edge) const
{
	return (nodes.first == t_edge.nodes.first)
&& (nodes.second == t_edge.nodes.second);
}

Edge Edge::CreateEdge(int t_first_node_id, int t_second_node_id)
{
	return Edge(t_first_node_id, t_second_node_id);
}


void Edge::UpdateMiddleNode(int t_middle_node)
{
	middle_node = t_middle_node;
}


