#pragma once
#include "DataStructures.h"
#include "HashCustom.h"
#include <set>
#include <vector>

class MeshLoader {
protected:
    std::vector<Node> nodes;
    std::vector<FiniteElement> finite_elements;
    std::vector<BoundaryFiniteElement> boundary;

private:
    std::array<double, 3> Get_Middle_Of_Edge(const Edge&);
    Node Get_Middle_Node(const Edge&);
    Node& Get_Node_By_ID(int);
    std::array<double, 3> Get_Coors_Node_By_ID(int);

public:
	virtual ~MeshLoader() = default;
	virtual void LoadMesh(const std::string&) = 0;

	const std::vector<Node>& Get_Node() const;
	const std::vector<FiniteElement>& Get_Finite_Elements() const;
	const std::vector<BoundaryFiniteElement>& Get_Boundary() const;

	std::vector<int> Get_Elements_By_ID(int t_node_id1, int t_node_id2, int t_node_id3) const;
	std::vector<int> Get_Elements_By_Edge(int t_node_id1, int t_node_id2) const;
	std::vector<int> Get_Elements_By_Material(int t_material_id) const;

	std::set<int> Get_Nodes_By_Surface(int t_surface_id) const;

	std::vector<int> Get_Boundary_By_Surface(int t_surface_id) const;

	void Insert_Middle_Nodes();
	
	std::vector<std::set<int>> Get_Adjacent_Nodes() const;

	
	std::vector<FiniteElement> Get_Nodes_By_Boundary(int t_bound) const;
	double Get_Area(const FiniteElement& finite_ele);
	double Get_Area_By_Nodes(Node& a, Node& b, Node& c);
	void AreaTriangle(const MeshLoader& Mesh, int border_id, const std::string& name_file);
};


